package edu.kit.ipd.descartes.adaptation.evaluation.weightingfunction;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.mm.adaptation.WeightedMetric;
import edu.kit.ipd.descartes.mm.adaptation.WeightingFunction;
import edu.kit.ipd.descartes.perfdatarepo.MetricType;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.Result;

public class WeightingFunctionHelper {
    
    static Logger logger = Logger.getLogger(WeightingFunctionHelper.class);

    /**
     * 
     * @param function
     * @param metricType
     * @return Returns the weight of the {@code meticType} of the given {@code function}.<br>
     *         Returns {@code Double.NaN} if the {@code metricType} was not found.
     */
    public static double getWeightForMetricType(WeightingFunction function, MetricType metricType) {
        EList<WeightedMetric> weightedMetrics = function.getWeightedMetrics();

        for (WeightedMetric weightedMetric : weightedMetrics) {
            if (EcoreUtil.equals(weightedMetric.getMetricType(), metricType))
                return weightedMetric.getWeight();
            else if (weightedMetric.getMetricType().getName().equals(metricType.getName())) {
                logger.debug("Could not find identical metric, using matching name instead");
                return weightedMetric.getWeight();
            }
        }

        return Double.NaN;
    }

    /**
     * Searches the {@link MetricValue} of the given {@link MetricType} in the {@link Result}
     * <code>resultBefore</code>
     * 
     * @param metricType
     *            The type of the metric to find
     * @param result
     *            The result to search
     * @return
     */
    public static double getValueForMetricType(MetricType metricType, Result result) {
        EList<MetricValue> metricValues = result.getMetricValues();

        for (MetricValue metricValue : metricValues) {
            if (sameMetricType(metricValue, metricType))
                return metricValue.getValue();
        }
        return Double.NaN;
    }

    public static boolean sameMetricType(MetricValue metricValue, MetricType metricType) {
//        return metricValue.getMetricType().equals(metricType);
        return (metricValue.getMetricType().getName().equals(metricType.getName())); 
//                && metricValue.getMetricType().getId().equals(metricType.getId()));
    }
}
