package edu.kit.ipd.descartes.adaptation.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;

public abstract class AbstractEcoreModelLoader {
	
    private ResourceSet resourceSet = null;
    static Logger logger = Logger.getLogger(AbstractEcoreModelLoader.class);
        
	protected ResourceSet getResourceSet() {
	    if (resourceSet == null)
	        throw new NullPointerException("INTITIALIZE RESOURCE SET FIRST! ");
        return resourceSet;
    }

	protected void setResourceSet(ResourceSet resourceSet) {
        this.resourceSet = resourceSet;
    }

	/**Implement this method to initialize your {@link ResourceSet}, e.g.,
	 * if you load your models in a standalone application.
	 * 
	 * @param resourceSet
	 */
	public abstract void initializeResourceSet(ResourceSet resourceSet);

	/**
	 * Loads the model specified by <code>uri</code>.
	 * 
	 * @param uri
	 * @return
	 * @throws IOException
	 */
    public EObject load(URI uri) {
        if (null == uri) {
            throw new IllegalArgumentException("Missing file URI.");
        }
        // Resource resource = resourceSet.createResource(fileURI);
        Resource resource = resourceSet.getResource(uri, true);
        try {
            resource.load(null);
            logger.info("Model from URI " + uri.toFileString() + " loaded.");
        } catch (IOException e) {
            logger.error("Error while loading model " + uri.toFileString(), e);
            e.printStackTrace();
        }
        EcoreUtil.resolveAll(resourceSet);
        return resource.getContents().get(0);
    }
	
    /**
     * Persists the specified <code>obj</code> at the given
     * <code>uri</code>.
     * 
     * @param uri
     * @param obj
     * @throws IOException
     */
	public void save(URI uri, EObject obj) throws IOException {
        if (null == uri) {
            throw new IllegalArgumentException("Missing file URI.");
        }
        Resource resource = resourceSet.createResource(uri);
        resource.getContents().add(obj);
        Map<String, Boolean> options = new HashMap<String, Boolean>();
        options.put(XMIResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE);
        options.put(XMIResource.OPTION_SAVE_ONLY_IF_CHANGED_MEMORY_BUFFER, Boolean.TRUE);
        resource.save(options);
    }
	
	/** Convenience method to create an URI.
	 *
	 * @param projectname
	 * @param filename
	 * @param filename_extension
	 * @return
	 */
    public static URI createFileURI(String projectname, String filename, String filename_extension) {
        if (null == projectname) {
            throw new IllegalArgumentException("Missing project name.");
        }
        if (null == filename) {
            throw new IllegalArgumentException("Missing file name.");
        }
        if (null == filename_extension) {
            throw new IllegalArgumentException("Missing file name extension.");
        }
        /* TODO check for null pointer exceptions */
        IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
        IProject project = root.getProject(projectname);
        String projectPath = project.getLocationURI().getPath();
        URI fileURI = URI.createFileURI(projectPath);
        return fileURI.appendSegment(filename).appendFileExtension(filename_extension);
    }

}
