package edu.kit.ipd.descartes.adaptation.model.dmm;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import edu.kit.ipd.descartes.adaptation.model.AbstractEcoreModelLoader;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationPackage;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationProcess;
import edu.kit.ipd.descartes.mm.adaptationpoints.AdaptationpointsPackage;
import edu.kit.ipd.descartes.mm.containerrepository.ContainerrepositoryPackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.DistributedDataCenter;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapePackage;

public class AdaptationProcessModelLoader extends AbstractEcoreModelLoader {

    public static final EPackage[] DMM_EPACKAGES = new EPackage[] { 
            ResourcelandscapePackage.eINSTANCE,
            AdaptationpointsPackage.eINSTANCE, 
            ContainerrepositoryPackage.eINSTANCE, 
            AdaptationPackage.eINSTANCE };

    private static AdaptationProcessModelLoader instance = null;

    public static AdaptationProcessModelLoader getInstance() {
        if (instance == null) {
            instance = new AdaptationProcessModelLoader();
        }
        return instance;
    }

    public AdaptationProcessModelLoader() {
        
        initializeResourceSet(new ResourceSetImpl());
    }

    public void saveAll() throws IOException {

        EList<Resource> resources = getResourceSet().getResources();
        Map<String, String> options = new HashMap<String, String>();
        options.put(XMIResource.OPTION_SCHEMA_LOCATION, Boolean.TRUE.toString());
        options.put(XMIResource.OPTION_SAVE_ONLY_IF_CHANGED_MEMORY_BUFFER, Boolean.TRUE.toString());
        options.put(XMIResource.OPTION_PROCESS_DANGLING_HREF, XMIResource.OPTION_PROCESS_DANGLING_HREF_THROW);

        for (Resource res : resources) {
            res.save(options);
        }
    }

    @Override
    public void initializeResourceSet(ResourceSet resourceSet) {
        setResourceSet(resourceSet);
        
        /* Register the default resource factory -- only needed for stand-alone! */
        getResourceSet().getResourceFactoryRegistry().getExtensionToFactoryMap()
                .put(Resource.Factory.Registry.DEFAULT_EXTENSION, new XMIResourceFactoryImpl());

        /* Register also the packages needed for the DMM meta model */
        for (EPackage ePackage : DMM_EPACKAGES) {
            getResourceSet().getResourceFactoryRegistry().getExtensionToFactoryMap().put(ePackage.getNsURI(), ePackage);
        }
    }

    public static DistributedDataCenter getDistributedDataCenter() {
        for (Iterator<Resource> iter = getInstance().getResourceSet().getResources().iterator(); iter.hasNext();) {
            Resource resource = (Resource) iter.next();
            if (resource.getContents().get(0).eClass().getClassifierID() == ResourcelandscapePackage.DISTRIBUTED_DATA_CENTER)
                return (DistributedDataCenter) resource.getContents().get(0);
        }
        return null;
    }

    @Override
    public AdaptationProcess load(URI uri) {
        return (AdaptationProcess) super.load(uri);
    }
}
