package edu.kit.ipd.descartes.adaptation.model.dmm.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.core.AdaptableEntity;
import edu.kit.ipd.descartes.core.NamedElement;
import edu.kit.ipd.descartes.mm.adaptation.AdaptationScope;
import edu.kit.ipd.descartes.mm.containerrepository.ContainerTemplate;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ConfigurationSpecification;
import edu.kit.ipd.descartes.mm.resourceconfiguration.ResourceconfigurationPackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.Container;
import edu.kit.ipd.descartes.mm.resourcelandscape.ResourcelandscapePackage;
import edu.kit.ipd.descartes.mm.resourcelandscape.RuntimeEnvironment;

public class DmmModelActionHelper {

    /**
     * 
     * @param aEntity
     * @return the {@link EObject} of the model instance that contains {@code aEntity}
     */
    public static EObject findParentConfiguredInstance(AdaptableEntity aEntity) {
        
        ConfigurationSpecification c = null;
        
        if (aEntity.eClass().getClassifierID() == ResourceconfigurationPackage.NUMBER_OF_PARALLEL_PROCESSING_UNITS)
            c = (ConfigurationSpecification) aEntity.eContainer().eContainer();
        else if (aEntity.eClass().getClassifierID() == ResourceconfigurationPackage.PASSIVE_RESOURCE_CAPACITY)
            c = (ConfigurationSpecification) aEntity.eContainer();
        else
            return null;
               
        return c.eContainer();
    }
    
    /**
     * Query for a {@link Container} with similar specs
     * 
     * @param candidates
     * @param comparatorContainer
     * @return
     */
    public static List<Container> filterContainersWithSimilarSpecs(List<Container> candidates,
            Container comparatorContainer) {

        List<Container> resultList = new ArrayList<Container>();
        
        for (Container c : candidates) {

            /*
             * Make sure we find a RuntimeEnvironment with 1. a DIFFERENT ID 2. similar properties
             * and 3. name must contain COPY_MARK to make sure we remove only a copy
             */
            if (c.getId() != comparatorContainer.getId() 
                    && c.getName().contains(DmmModelChanger.COPY_MARK) 
                    && EcoreUtil.equals(comparatorContainer.getConfigSpec(), c.getConfigSpec())
                    && EcoreUtil.equals(comparatorContainer.getTemplate(), c.getTemplate())) {
                resultList.add(c);
            }
        }
        return resultList;
    }
    
    /**
     * Query for a {@link RuntimeEnvironment} that 
     * has been copied (contains {@link DmmModelChanger.COPY_MARK}) and refers to
     * the same ContainerTemplate.
     * 
     * @param candidates
     * @param comparatorRE
     * @return
     */
    public static RuntimeEnvironment filterCopiedRuntimeEnvironment(List<RuntimeEnvironment> candidates,
            RuntimeEnvironment comparatorRE) {

        for (RuntimeEnvironment re : candidates) {

            if (re.getId() != comparatorRE.getId() 
                    && re.getName().contains(DmmModelChanger.COPY_MARK) 
//                    && EcoreUtil.equals(comparatorRE.getConfigSpec(), re.getConfigSpec())
//                    && EcoreUtil.equals(comparatorRE.getContainedIn(), re.getContainedIn())
                    && EcoreUtil.equals(comparatorRE.getTemplate(), re.getTemplate())) {
                return re;
            }
        }
        return null;
    }
    
    public static NamedElement findEntityWithName(NamedElement element, String name) {
       
        if(element.getName().equals(name))
            return element;
        else {
            NamedElement result = null;
            for (Iterator<EObject> iterator = element.eContents().iterator(); iterator.hasNext();) {
                NamedElement nextElement = (NamedElement) iterator.next();
                result = findEntityWithName(nextElement, name);
                if (result != null)
                    return result;
            }
            return result;
        }
    }
    
    /** Queries for all Container instances that have a similar configuration
     *  specification as Container <code>c</code>. Similar configuration 
     *  specification meas that ContainerTemplate and ConfigurationSpecification
     *  are equal.
     * 
     * @param c 
     * @return A list of Container instances
     */
    public static List<Container> queryContainersWithSimilarSpecs(Container c) {
           List<Container> result = queryContainersWithSimilarTemplate(c);
           return filterContainersWithSimilarSpecs(result, c);
    }

    /**Queries for a RuntimeEnvironment whose ContainerTemplate is similar to the
     * given RuntimeEnvironment <code>re</code> and whose capacity is below <code>maxValue</code>.
     * 
     * @param re
     * @param maxValue
     * @return
     */
    public static RuntimeEnvironment queryUnfilledRuntimeEnvironment(RuntimeEnvironment re, double maxValue) {
        //TODO SEAMS specific helper function! Generalize!
        String queryExpression = "RuntimeEnvironment.allInstances()->select( re | re.template = self.template )->select(re | re.configSpec.oclAsType(resourceconfiguration::ActiveResourceSpecification).processingResourceSpecifications->forAll(nrOfParProcUnits.number < "+maxValue+"))";
        List<EObject> resultList = new ArrayList<EObject>(
                OclEvaluationHelper.query(re, queryExpression, re.eClass()));
        
        if (resultList.size()>0)
            return (RuntimeEnvironment) resultList.get(0);
        else
            return null;
    }
    
    public static RuntimeEnvironment queryNotEmptyRuntimeEnvironment(RuntimeEnvironment re, double minValue) {
        // TODO SEAMS specific helper function! Generalize!
        String queryExpression = "RuntimeEnvironment.allInstances()->select( re | re.template = self.template and not re.configSpec->isEmpty() )->select(re | re.configSpec.oclAsType(resourceconfiguration::ActiveResourceSpecification).processingResourceSpecifications->forAll(nrOfParProcUnits.number > "+minValue+"))";
        List<EObject> resultList = new ArrayList<EObject>(
                OclEvaluationHelper.query(re, queryExpression, re.eClass()));
        
        if (resultList.size()>0)
            return (RuntimeEnvironment) resultList.get(0);
        else
            return null;
    }

    /**Queries for all Container instances referring to the same 
     * ContainerTemplate as the given container.
     * 
     * @param container The container to look for
     * @return All Container Instances referring to the same ContainerTemplate
     */
    public static List<Container> queryContainersWithSimilarTemplate(Container container) {
        String queryExpression = "RuntimeEnvironment.allInstances()->select( re | re.template = self.template and re.id <> self.id)";
        List<EObject> tempList = new ArrayList<EObject>(
                OclEvaluationHelper.query(container, queryExpression, container.eClass()));
        List<Container> resultList = new ArrayList<Container>(tempList.size());
        for (Iterator<EObject> iterator = tempList.iterator(); iterator.hasNext();) {
            Container temp = (Container) iterator.next();
            resultList.add(temp);
        }
        return resultList;
    }

    
    /**Filter the given {@code containerList} for RuntimeEnvironments and cast 
     * them to {@link RuntimeEnvironment}
     * @param containerList
     * @return a list of RuntimeEnvironments
     */
    public static List<RuntimeEnvironment> filterAndConvertRuntimeEnvironment(List<Container> containerList) {
        List<RuntimeEnvironment> result = new ArrayList<RuntimeEnvironment>();
        
        for (Container container : containerList) {
            if (container.eClass().getClassifierID()  == ResourcelandscapePackage.RUNTIME_ENVIRONMENT)
                result.add((RuntimeEnvironment) container);
        }
        
        return result;
    }
    
    /**Find a container that refers to the given {@link ContainerTemplate} and that suits the
     * given {@link AdaptationScope}.
     * 
     * @param containerTemplate
     * @param adaptationScope
     * @return
     */
    public static Container findSuitableContainer(ContainerTemplate containerTemplate, AdaptationScope adaptationScope) {
        List<Container> containers = containerTemplate.getReferringContainers();
        Random randomGenerator = new Random();

        switch (adaptationScope) {
        case RANDOM:
            return containers.get(randomGenerator.nextInt(containers.size()));
        default:
            return null;
        }
    }
    
}
