package edu.kit.ipd.descartes.adaptation.model.dmm.util;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ocl.OCL;
import org.eclipse.ocl.ParserException;
import org.eclipse.ocl.Query;
import org.eclipse.ocl.ecore.Constraint;
import org.eclipse.ocl.ecore.EcoreEnvironmentFactory;
import org.eclipse.ocl.expressions.OCLExpression;
import org.eclipse.ocl.helper.OCLHelper;

/**
 * @author nhuber
 *
 */
public class OclEvaluationHelper {
	
	private static Logger logger = Logger.getLogger(OclEvaluationHelper.class);

	/**
	 * @param checkedObject the checked object
	 * @param constraint the constraint to evaluate on the object
	 * @param context the {@link EClass} of the given object
	 * @return
	 *     {@code true} if check was successful
	 */
	public static boolean evaluateOclConstraint(EObject checkedObject, String constraint, EClass context) {
		Constraint invariant = null;
		OCL<?, EClassifier, ?, ?, ?, ?, ?, ?, ?, Constraint, EClass, EObject> ocl;
		ocl = OCL.newInstance(EcoreEnvironmentFactory.INSTANCE);
		OCLHelper<EClassifier, ?, ?, Constraint> helper = ocl.createOCLHelper();
		
		helper.setContext(context);
		
		try {
			invariant = helper.createInvariant(constraint);
		} catch (ParserException e) {
			logger.error("Error parsing OCL string '" + constraint + "'", e);
			return false;
		}
		
		Query<EClassifier, EClass, EObject> eval = ocl.createQuery(invariant);
		return eval.check(checkedObject);		
	}

	
	
	/**
	 * Executes the query on a given object.
	 * 
	 * @param obj the model instance the query is executed on
	 * @param queryExpression the OCL query
	 * @param context the eClass of the model instance
	 * @return
	 */
	public static Set<EObject> query(EObject obj, String queryExpression, EClassifier context) {
	    OCL<?, EClassifier, ?, ?, ?, ?, ?, ?, ?, Constraint, EClass, EObject> ocl;
	    ocl = OCL.newInstance(EcoreEnvironmentFactory.INSTANCE);
	    OCLHelper<EClassifier, ?, ?, Constraint> helper = ocl.createOCLHelper();

	    helper.setContext(context);
	    
	    OCLExpression<EClassifier> query;
        try {
            query = helper.createQuery(queryExpression);
        } catch (ParserException e) {
            logger.error("Error parsing OCL query '" + queryExpression + "'", e);
            return null;
        }

	    // create a Query to evaluate our query expression
	    Query<EClassifier, EClass, EObject> queryEval = ocl.createQuery(query);

	    Object result = queryEval.evaluate(obj);

        if (result.toString().contains("invalid"))
            return null;
                
        @SuppressWarnings("unchecked")  
        Set<EObject> resultList = (Set<EObject>) result;

        return resultList;
	}
}
