package edu.kit.ipd.descartes.adaptation.model.perfdatarepo;

import static edu.kit.ipd.descartes.adaptation.model.perfdatarepo.PerfDataRepoConverter.convertObservableImpact;
import static edu.kit.ipd.descartes.adaptation.model.perfdatarepo.PerfDataRepoConverter.convertResult;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.URI;

import edu.kit.ipd.descartes.adaptation.model.AbstractEcoreModelLoader;
import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.Result;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class ObservationRepositoryHandler implements IPerformanceDataRepositoryHandler {

    private static Logger logger = Logger.getLogger(ObservationRepositoryHandler.class);
    private AbstractEcoreModelLoader modelLoader = null;
    private ObservationRepository repository = null;
    private int iteration = 0;
    
    public ObservationRepositoryHandler() {
        modelLoader = new ObservationRepositoryModelLoader();
        logger.debug("ObservationRepositoryHandler loaded.");
    }
    
    @Override
    public Impact getLatestImpact() {
        return convertObservableImpact(repository.getLatestImpact());
    }

    @Override
    public Impact getImpactAt(int position) {
        iteration++; //This is a hack, see getCurrentSystemState()
        if (position > repository.getImpacts().size() - 1)
            return null;
        return convertObservableImpact(repository.getImpacts().get(position));
    }

    @Override
    public void load(String modelFileUri) {
        repository = (ObservationRepository) modelLoader.load(URI.createFileURI(modelFileUri));
    }

    @Override
    public Result getCurrentSystemState() {
        return convertResult(repository.getObservations().get(iteration));
    }

}
