package edu.kit.ipd.descartes.adaptation.model.perfdatarepo;

import java.util.ArrayList;
import java.util.Collection;

import edu.kit.ipd.descartes.perfdatarepo.Impact;
import edu.kit.ipd.descartes.perfdatarepo.MeasuredEntity;
import edu.kit.ipd.descartes.perfdatarepo.MetricType;
import edu.kit.ipd.descartes.perfdatarepo.MetricValue;
import edu.kit.ipd.descartes.perfdatarepo.PerfdatarepoFactory;
import edu.kit.ipd.descartes.perfdatarepo.Result;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.repository.ObservableImpact;
import edu.kit.ipd.descartes.ql.models.repository.Observation;

public class PerfDataRepoConverter {
    
    public static Impact convertObservableImpact(ObservableImpact o) {
        Impact impact = PerfdatarepoFactory.eINSTANCE.createImpact();
        impact.setId(o.getIdentifier());
        impact.setName("ObservableImpact (ID: " + o.getIdentifier() + ")");
        impact.setBefore(convertResult(o.getBefore()));
        impact.setAfter(convertResult(o.getAfter()));
        return impact;
    }

    public static Result convertResult(Observation o) {
        Result result = PerfdatarepoFactory.eINSTANCE.createResult();
        result.setId(o.getIdentifier());
        result.setName("Observation (ID: " + o.getIdentifier() + ")");
        Collection<MetricValue> values = convertMapping(o.getObservation());
        result.getMetricValues().addAll(values);
        return result;
    }

    public static Collection<MetricValue> convertMapping(EntityMapping observation) {
        Collection<MetricValue> values = new ArrayList<MetricValue>();
        
        for (Resource r : observation.getResources()) {
            values.addAll(convertProbes(r));        
        }
        
        for (Service s : observation.getServices()) {
            values.addAll(convertProbes(s));   
        }
        
        return values;
    }

    private static Collection<MetricValue> convertProbes(Entity e) {
        Collection<MetricValue> values = new ArrayList<MetricValue>();
        MetricType mType = PerfdatarepoFactory.eINSTANCE.createMetricType();
        mType.setName(e.getProbes().get(0).getMetricName());
        mType.setId(e.getProbes().get(0).getMetricName());
        MeasuredEntity entity = PerfdatarepoFactory.eINSTANCE.createMeasuredEntity();
        entity.setName(e.getAlias());
        entity.setId(e.getIdentifier());

        for (Probe p : e.getProbes()) {
            DecimalResult r = (DecimalResult) p;
            MetricValue value = PerfdatarepoFactory.eINSTANCE.createMetricValue();
            value.setMeasuredEntity(entity);
            value.setName(e.getAlias() + ": " + r.getMetricName());
            value.setMetricType(mType);
            value.setValue(r.getValue().doubleValue());
            values.add(value);
        }
        
        return values;
    }

}
